/*

This example shows how to use the DRAG and DROP clauses of
DC* commands for dragging and dropping operations

This samples supports dragging between objects of the same
window or different windows and also dragging from
Windows explorer to the Xbase++ application.

Each DC* command has a DRAG clause and a DROP clause.

Syntax: DRAG <bDrag> [DIALOG <bDialog>]
        DROP <bDrop> [CURSOR <nCursor>]


<bDrag> is a code block to evaluate when the Left Mouse button
has been depressed for more than 1/4 second or has been held
and dragged.  A pointer to the object will be passed to the
code block.  The code block should return a value to capture
from the object.  This value will later be dropped into another
object.

<bDialog> is a code block to evaluate for creating any special
dialog that will be displayed and dragged with the mouse.
The 3 standard values of mp1, mp2, and oXbp are passed to the
code block.

<bDrop> is a code block to evaluate when the Left Mouse button
has been lifted (up) in the object.  A pointer to the object
and the dropped value are passed to the code block.

<nCursor> is the pointer to use when the mouse is moved over
the drop object.  The default is POINTER_DATADROP_COPY.

*/

#INCLUDE "dcdialog.CH"
#INCLUDE "dctree.CH"
#INCLUDE "appevent.CH"
#INCLUDE "dccursor.CH"

FUNCTION Main()

Thread():new():start({||Dialog2()})
Sleep(200)
Thread():new():start({||Dialog1()})
Sleep(50)
Thread():new():start({||Dialog3()})

SetAppWindow(AppDeskTop())

Sleep(200)
DO WHILE !Empty(AppDeskTop():childList())
  Sleep(1)
ENDDO

RETURN nil

* -----------

FUNCTION Dialog1()

LOCAL GetList[0], oBrowse, aDir, GetOptions, i, bSort

aDir := Directory('..\..\bitmaps\*.*')

FOR i := 1 TO Len(aDir)
  aDir[i,1] := '..\..\bitmaps\' + aDir[i,1]
NEXT

@ 0,0 dcpushbuttonxp caption "But 1" title "B1" size 15,2 ;
		DRAG {|oXbp| DC_GetProperty( oXbp, cGETLIST_TITLE ) } ;
		DROP {|oXbp,a| (wtf DC_GetProperty( oXbp, cGETLIST_TITLE ),a) }

@ 0,25 dcpushbuttonxp caption "But 2" title "B2" size 15,2 ;
		DRAG {|oXbp| DC_GetProperty( oXbp, cGETLIST_TITLE ) } ;
		DROP {|oXbp,a| (wtf DC_GetProperty( oXbp, cGETLIST_TITLE ),a) }
		
@ 3,0 DCBROWSE oBrowse ;
      DATA aDir ;
      SIZE 40,20 FIT ;
      PRESENTATION DC_BrowPres()

DCBROWSECOL ELEMENT 1 PARENT oBrowse HEADER 'File Name' WIDTH 25 ;
     SORT {||ASort(aDir,,,{|a,b|a[1]<b[1]})} LEFTBUTTON ;
     DRAG {|o|{Eval(o:dataLink)}} ;
       DIALOG {|mp1,mp2,oXbp|DragDialog(mp1,mp2,oXbp)} ;

DCBROWSECOL ELEMENT 2 PARENT oBrowse HEADER 'File Size' ;
     SORT {||ASort(aDir,,,{|a,b|a[2]<b[2]})} LEFTBUTTON

DCBROWSECOL ELEMENT 3 PARENT oBrowse HEADER 'File Date'

DCBROWSECOL ELEMENT 4 PARENT oBrowse HEADER 'File Time'

DCGETOPTIONS RESIZE RESIZEDEFAULT DCGUI_RESIZE_RESIZEONLY

DCREAD GUI FIT TITLE 'Drag Drop Dialog 1' OPTIONS GetOptions

RETURN nil

* -----------

FUNCTION Dialog2()

LOCAL GetList[0], cText := '', aFiles[0], oBrowse, oXbp, ;
      cFileName := Space(50), oStaticBitmap1, oStaticBitmap2

@ 0,0 DCSAY 'File' GET cFileName SAYSIZE 0 SAYBOTTOM ;
        GETSIZE 33 ;
        DATALINK {||nil} ;
        GETDROP {|o,a|o:_assign(Pad(a[1],Len(o:buffer)))}

@ 2, 0 DCSTATIC TYPE XBPSTATIC_TYPE_TEXT CAPTION '..\..\bitmaps\explarge.bmp' SIZE 39,8 ;
        DRAG {|o|{o:caption}} DIALOG {|mp1,mp2,oXbp|DragDialog(mp1,mp2,oXbp)} ;
        DROP {|o,a|o:setCaption(a[1]),o:invalidateRect()} ;
        COLOR GRA_CLR_BLACK, GRA_CLR_WHITE ;
        OBJECT oStaticBitmap1

DCBITMAP {||oStaticBitmap1:caption} PARENT oStaticBitmap1 AUTOSCALE CENTER

@ 11, 0 DCSTATIC TYPE XBPSTATIC_TYPE_TEXT CAPTION '.\Harley.jpg' SIZE 39,9 ;
        DRAG {|o|{o:caption}} DIALOG {|mp1,mp2,oXbp|DragDialog(mp1,mp2,oXbp)} ;
        DROP {|o,a|o:setCaption(a[1]),o:invalidateRect()} ;
        COLOR GRA_CLR_BLACK, GRA_CLR_WHITE ;
        OBJECT oStaticBitmap2

DCBITMAP {||oStaticBitmap2:caption} PARENT oStaticBitmap2 AUTOSCALE CENTER

@ 0, 40 DCMULTILINE cText SIZE 39,20 FONT '10.Courier New Bold' ;
      drop {|o,a| (wtf o,a) , ""}

//         DROP {|o,a|oXbp := o, AEval(a,{|x|oXbp:setData(oXbp:editBuffer() + x + Chr(13)+Chr(10))})}

@ 0, 80 DCBROWSE oBrowse DATA aFiles SIZE 10,20 FIT ;
        PRESENTATION DC_BrowPres()

DCBROWSECOL ELEMENT 1 HEADER 'File Name' WIDTH 30 PARENT oBrowse ;
        DROP {|o,a|AEval(a,{|x|AAdd(aFiles,{x})}), ;
                   oBrowse:refreshAll()} CURSOR POINTER_DATADROP_ASSIGN ;
        DRAG {|o|{Eval(o:dataLink)}} ;
          DIALOG {|mp1,mp2,oXbp|DragDialog(mp1,mp2,oXbp)} ;

DCREAD GUI FIT TITLE 'Drag Drop Dialog 2'

RETURN nil

* -----------

FUNCTION Dialog3()

LOCAL GetList[0], GetOptions, aDir, oTree, i

aDir := Directory()

@ 0, 0 DCTREEROOT OBJECT oTree SIZE 30,20 ;
       ALWAYSSHOWSELECTION ;
        DRAG {|o,x|{o:getData():caption}} ;
          DIALOG {|mp1,mp2,oXbp|DragDialog(mp1,mp2,oXbp)} ;
        DROP {|o,a|AEval(a,{|x|oTree:rootItem:AddItem(x)})} ;
        COLOR GRA_CLR_BLACK, GRA_CLR_WHITE

FOR i := 1 TO Len(aDir)
  DCTREEITEM CAPTION aDir[i,1] PARENT oTree
NEXT

DCGETOPTIONS RESIZE RESIZEDEFAULT DCGUI_RESIZE_RESIZEONLY

DCREAD GUI FIT TITLE 'Drag Drop Dialog 3' OPTIONS GetOptions

RETURN nil

* -----------

PROC appsys
RETURN

* -----------

STATIC FUNCTION DragDialog( mp1, mp2, oXbp )

LOCAL GetList[0], GetOptions, oDialog, aPos, cText

IF oXbp:isDerivedFrom('XbpColumn')
  cText := Trim(Eval(oXbp:dataLink))
ELSEIF oXbp:isDerivedFrom('XbpStatic')
  cText := Trim(oXbp:caption)
ELSEIF oXbp:isDerivedFrom('XbpTreeView')
  cText := Trim(oXbp:getData():caption)
ENDIF

@ 2,0 DCSTATIC TYPE XBPSTATIC_TYPE_BITMAP SIZE 10,1.2 ;
   CAPTION "..\..\bitmaps\file1.Bmp"

@ DCGUI_ROW, DCGUI_COL DCSAY cText ;
   COLOR GRA_CLR_BLACK, GRA_CLR_YELLOW ;
   FONT '10.Courier' SAYSIZE 0,1.2 ;
   OPTION XBPSTATIC_TEXT_LEFT + XBPSTATIC_TEXT_VCENTER

DCGETOPTIONS NOTITLEBAR ORIGIN XBPDLG_ORIGIN_MOUSE ;
   FITPAD 0 COLOR { 240, 240, 255 } ;
   BORDER XBPDLG_NO_BORDER ;
   HIDE ;
   NOTASKLIST

DCREAD GUI FIT OPTIONS GetOptions PARENT @oDialog EXIT NOAUTORESTORE

aPos := DC_CalcAbsolutePosition(mp1,oXbp)
aPos[2] -= (oDialog:currentSize()[2] + 30)
oDialog:setPos( aPos )
oDialog:show()

RETURN oDialog





